//----------------------------------------------------------------------------

//LEVEL
//  

//----------------------------------------------------------------------------

#include "Main.h"
#include "Usonic.h"
#include "Sound.h"

//-----------------    : -----------------

// 0    -  40 .
// 1      .

//    ( Usonic_Start) 
// :

//1.  .
//2. Timer1 = 0.
//3. Timer0 = 0.
//4. OC0B toggle on compare.
//5. OCR1A =    N_USONIC   F_USONIC.
//6.   TIMER1_COMPA.
//7. Count = MAX_COUNT.
//8. State = U_START.
//9.  .

//    OC0B     N_USONIC
//  F_USONIC.    
//TIMER1_COMPA.     :

//1. OC0B clear on compare.
//2. OCR1A = OCR1A + DEAD_TIME.
//3. State = U_DEAD.

//    DEAD_TIME.    
//  TIMER1_COMPA.     :

//1.   TIMER1_CAPT  TIMER1_COMPA.
//2.       (Pin_UIN = 1),  State = U_WAIT.
//3.     (Pin_UIN = 0),   
//   ,  Count = 0,   TIMER1_CAPT 
//   TIMER1_COMPA, State = U_STOP,    Count = 0.

//      ,   
//    TIMER1_CAPT.   
// :

//1. Count = ICR1.
//2.   TIMER1_CAPT  TIMER1_COMPA.
//3. State = U_STOP,    Count = ICR1.

//     ,   
// TIMER1_COMPA (     
// 1,     4    16 ).
//    :

//1.   TIMER1_CAPT  TIMER1_COMPA.
//2. State = U_STOP,    Count = MAX_COUNT.

//   ( State   U_STOP)
//    Count  :
//Level [mm] = Count * SPEED [cm/s] / F_CLK_KHZ / 1000 * 10 / 2
// 1000  - ,     /,
//  -  .  1 / F_CLK_KHZ     
//,         1000.
// 10 -     .
//   2      -:
//   .

//----------------------------- : -----------------------------------

#define F_CLK_KHZ (F_CLK * 1000) // , 
#define F_USONIC  40.0   //  , 
#define N_USONIC  7      //  
#define DEAD_TIME 1500   // , 
#define INIT_TM   1000   //  , 
#define POLL_TM   200    // , 
#define REL_DEL   10     //  , 
#define SPEED     34320  // , /
#define MAX_COUNT 0xFFFF //  
#define MAX_LEVEL 3000   // 
#define YEL_LEVEL 1000   // 
#define RED_LEVEL  500   // 

#define REL_TMV (REL_DEL * 1000 / POLL_TM)

enum { U_STOP, U_START, U_DEAD, U_WAIT };

//----------------------------- : ----------------------------------

volatile static char State;
static unsigned int Level;
static unsigned int Count;
static unsigned int PollTimer;
static unsigned int RelTimer;
static bool Relay;

//----------------------------- : -------------------------------------

#pragma vector = TIMER1_CAPT_vect
__interrupt void Capture(void);
#pragma vector = TIMER1_COMPA_vect
__interrupt void Compare(void);
void Usonic_Start(void);
unsigned int Usonic_ConvLevel(unsigned int c);
void Usonic_ShowLevel(unsigned int v);

//-----------------------  : --------------------------

void Usonic_Init(void)
{
  //Timer 0 init:
  //timer frequency - F_USONIC * 2:
  OCR0A = (char)(F_CLK_KHZ / F_USONIC / 2 - 0.5);
  //clear OC0B on compare, CTC:
  TCCR0A = (1 << COM0B1) | (1 << WGM01);
  //ICP noise canceler, ICP falling edge, CK/1:
  TCCR0B = (1 << ICNC1) | (1 << CS00);
  //Timer 1 init:
  //CK/1:
  TCCR1B = 1 << CS10;
  Count = MAX_COUNT;
  Level = 0; //MAX_LEVEL;
  State = U_STOP;
  RelTimer = 0;
  Relay = 0;
  PollTimer = INIT_TM; // 
  Port_LED3_0; Port_LED2_0; Port_LED1_0; //  
}

//------------------------   : ---------------------------

void Usonic_Start(void)
{
  char si = __save_interrupt();
  __disable_interrupt();       // 
  TCNT1 = 0;
  TCNT0 = 0;
  TCCR0A = (1 << COM0B0) | (1 << WGM01); //toggle OC0B
  OCR1A = (int)(F_CLK_KHZ / F_USONIC * N_USONIC);
  TIFR1 = 1 << OCF1A;   //OC1A interrupt flag clear
  TIMSK1 = 1 << OCIE1A; //OC1A interrupt enable
  Count = MAX_COUNT;
  State = U_START;
  __restore_interrupt(si);     // 
}

//--------------------------  : -------------------------------

void Usonic_Exe(bool t)
{
  if(t)
  {
    if(PollTimer) PollTimer--;
    else
    {
      PollTimer = POLL_TM;
      unsigned int lev = Usonic_ConvLevel(Count);
      if(lev < MAX_LEVEL) Level = lev;
      Usonic_Start();
      Usonic_ShowLevel(Level);
    }
  }
}

//------------------------   : --------------------------

#pragma vector = TIMER1_COMPA_vect
__interrupt void Compare(void)
{
  switch(State)
  {
  case U_START:
    TCCR0A = (1 << COM0B1) | (1 << WGM01); //clear OC0B
    OCR1A += (int)(DEAD_TIME * F_CLK); //+ DEAD_TIME
    State = U_DEAD;
    break;
  case U_DEAD:
    TIFR1 = (1 << ICF1) | (1 << OCF1A);    //IC1 and OC1A interrupts flags clear
    TIMSK1 = (1 << ICIE1) | (1 << OCIE1A); //IC1 and OC1A interrupts enable
    if(Pin_UIN)
    {
      State = U_WAIT;
      break;
    }
    Count = 0;
  case U_WAIT:
    TIMSK1 = 0; //IC1 and OC1A interrupts disable
    State = U_STOP;
    break;
  }
}

//-------------------------   : ---------------------------

#pragma vector = TIMER1_CAPT_vect
__interrupt void Capture(void)
{
  Count = ICR1;
  TIMSK1 = 0; //IC1 and OC1A interrupts disable
  State = U_STOP;
}

//--------------------------  : ------------------------------

//Level [mm] = Count * SPEED [cm/s] / F_CLK_KHZ / 1000 [ms->s] *
//10 [cm->mm] / 2 [distance rx+tx]

unsigned int Usonic_ConvLevel(unsigned int c)
{
  unsigned int v = MAX_LEVEL;
  if(c != MAX_COUNT)
    v = (unsigned long)c * SPEED / (unsigned long)(F_CLK_KHZ * 200);
  return(v);
}

//--------------------------  : -------------------------------

void Usonic_ShowLevel(unsigned int v)
{
  //"" :
  if(v < RED_LEVEL)
  {
    Port_LED3_1; Port_LED2_1; Port_LED1_0; Sound_Start(D_TICK);
    if(RelTimer < REL_TMV) RelTimer++;
    if(!Relay && RelTimer == REL_TMV)
    {
      Port_REL_1;
      Relay = 1;
    }
  }
  else
  {
    if(RelTimer > 0) RelTimer--;
    if(Relay && RelTimer == 0)
    {
      Port_REL_0;
      Relay = 0;
    }
    //"" :
    if (v < YEL_LEVEL) { Port_LED3_1; Port_LED2_0; Port_LED1_1; }
    //"" :
    else { Port_LED3_0; Port_LED2_1; Port_LED1_1; }
  }
}

//---------------------------  : ---------------------------------

unsigned int Usonic_GetLevel(void)
{
  return(Level);
}

//----------------------------------------------------------------------------
